from pytezergis.spatial.envelope import Envelope
from pytezergis.spatial.latlng import Latlng

class Bbox:
    def __init__(self, p1:Latlng, p2:Latlng):
        """
        Classe contendo duas coordenadas representando as extremidades de um poligono

        Args:
            p1 (tuple(Latlng), Envelop)
            p2 (Latlng, optional)
        """
        self.p1 = p1
        self.p2 = p2

    @staticmethod
    def createFromList(latlngs:tuple):
        p1 = Latlng(latlngs[1], latlngs[0])
        p2 = Latlng(latlngs[3], latlngs[2])

        return Bbox(p1, p2)

    @staticmethod
    def createFromEnvelope(env:Envelope):
        p1 = env.south_west()
        p2 = env.north_east()

        return Bbox(p1, p2)

    def first(self):
        if self.p1.lng < self.p2.lng:
            return self.p1

        return self.p2

    def last(self):
        if self.p1.lng < self.p2.lng:
            return self.p2
            
        return self.p1

    def toArray(self):
        first = self.first()
        last = self.last()

        return (first.lng, first.lat, last.lng, last.lat)

    def toEnvelope(self):
        first = self.first()
        last = self.last()

        p1 = Latlng(last.lat, first.lng)
        p2 = last
        p3 = Latlng(first.lat, last.lng)
        p4 = first

        return Envelope(p1, p2, p3, p4)
    
    def __str__(self) -> str:
        return self.toWkt()
    
    def toWkt(self):
        pass
        first = self.first()
        last = self.last()

        return "LINESTRING (%s %s, %s %s)" % (first.lng, first.lat, last.lng, last.lat)


class GeoMathUtils:
    @staticmethod
    def query(ds, ulx, uly, lrx, lry, querysize=0):
        geotran = ds.GetGeoTransform()

        rx = int((ulx - geotran[0]) / geotran[1] + 0.001)
        ry = int((uly - geotran[3]) / geotran[5] + 0.001)
        rxsize = int((lrx - ulx) / geotran[1] + 0.5)
        rysize = int((lry - uly) / geotran[5] + 0.5)

        wxsize, wysize = querysize, querysize
        wx = 0
        if rx < 0:
            rxshift = abs(rx)
            wx = int(wxsize * (float(rxshift) / rxsize))
            wxsize = wxsize - wx
            rxsize = rxsize - int(rxsize * (float(rxshift) / rxsize))
            rx = 0
        if rx+rxsize > ds.RasterXSize:
            wxsize = int(wxsize * (float(ds.RasterXSize - rx) / rxsize))
            rxsize = ds.RasterXSize - rx

        wy = 0
        if ry < 0:
            ryshift = abs(ry)
            wy = int(wysize * (float(ryshift) / rysize))
            wysize = wysize - wy
            rysize = rysize - int(rysize * (float(ryshift) / rysize))
            ry = 0
        if ry+rysize > ds.RasterYSize:
            wysize = int(wysize * (float(ds.RasterYSize - ry) / rysize))
            rysize = ds.RasterYSize - ry

        return (rx, ry, rxsize, rysize), (wx, wy, wxsize, wysize)



from pytezergis.spatial.wkt import Wkt
from pytezergis.spatial.latlng import Latlng
from pytezergis.spatial.polygon import Polygon

class GeoReference:
    def __init__(self, latlng:Latlng, tx, ty, width, height):
        self._latlng = latlng
        self._tx = tx
        self._ty = ty

        self.width = width
        self.height = height

        self.__load_bounds()

    def pivot(self):
        return self._latlng

    def transform(self):
        return (self._latlng.lng, self._tx, 0.0, self._latlng.lat, 0.0, self._ty)
    
    def latLngsToPoints(self, latlngs):
        points = []

        for latlng in latlngs:
            points.append(self.latLngToPoint(latlng))

        return points
    
    def pointsToLatlngs(self, points):
        latlngs = []

        for point in points:
            latlngs.append(self.pointToLatLng(point))

        return latlngs

    def pointsToPolygon(self, points):
        return Polygon(self.pointsToLatlngs(points))

    def polygonToPoints(self, polygon:Polygon):
        return self.latLngsToPoints(polygon.toLatlngs())

    def pointToLatLng(self, point):
        x, y = point
        rx = float(x / self.width)
        ry = float(y / self.height)

        rs = self.__lerp(self.tr, self.br, ry)
        ls = self.__lerp(self.tl, self.bl, ry)

        (lng, lat) = self.__lerp(ls, rs, rx)

        return Latlng(lat, lng)

    def latLngToPoint(self, latlng:Latlng):
        transform = self.transform()

        x = int((latlng.lng - transform[0]) / transform[1] + 0.001)
        y = int((latlng.lat - transform[3]) / transform[5] + 0.001)

        return x, y

    def __lerp(self, p1, p2, t):
        x = ((1 - t)*p1[0]) + (t*p2[0])
        y = ((1 - t)*p1[1]) + (t*p2[1])

        return (x, y)

    def __load_bounds(self):
        self.tl = self.__get_corner(0.0, 0.0)
        self.tr = self.__get_corner(self.width, 0.0)
        self.bl = self.__get_corner(0.0, self.height)
        self.br = self.__get_corner(self.width, self.height)
        
    def __get_corner(self, x, y):
        transform = self.transform()

        gx = transform[0] + transform[1] * x + transform[2] * y
        gy = transform[3] + transform[4] * x + transform[5] * y

        return (gx, gy)

    @staticmethod
    def createFromTransform(transform, width, height):
        return GeoReference(Latlng(transform[3], transform[0]), transform[1], transform[5], width, height)

    @staticmethod
    def createFromWkt(wkt, width, height):
        if isinstance(wkt, str):
            wkt = Wkt(wkt)

        e = wkt.envelope()
        p1 = e.pivot()
        p2 = e.south_east()

        tx = (p2.lng - p1.lng) / width
        ty = (p2.lat - p1.lat) / height

        return GeoReference(p1, tx, ty, width, height)
    
    def __str__(self) -> str:
        return "pivot = %s \ntx = %s \nty = %s \nwidth = %s \nheight = %s" % (self._latlng, self._tx, self._tx, self.width, self.height)


from osgeo import ogr

from pytezergis.raster.abstract import AbstractRaster

class Shapefile(AbstractRaster):
    def __init__(self, filename:str):
        super(Shapefile, self).__init__(filename)

    def open(self, filename:str):
        driver = ogr.GetDriverByName('ESRI Shapefile')
        return driver.Open(filename, 0)


import numpy as np

from osgeo import osr, gdal
from pytezergis.geomathutils import GeoMathUtils
from pytezergis.spatial.georeference import GeoReference
from pytezergis.spatial.extent import Extent
from pytezergis.spatial.envelope import Envelope
from pytezergis.spatial.bbox import Bbox

class RasterOrder:
    CRGB = 1
    RGBC = 2

class RasterType:
    BYTE = gdal.GDT_Byte
    UINT16 = gdal.GDT_UInt16
    INT16 = gdal.GDT_Int16

    @staticmethod
    def convertToNumpyType(type):
        if type == RasterType.BYTE:
            return np.uint8
        
        if type == RasterType.UINT16:
            return np.uint16
        
        if type == RasterType.INT16:
            return np.int16

class Raster:
    READ  = 0
    WRITE = 1
    
    def __init__(self, filename, mode=READ):
        raster = self._raster = self.open(filename, mode)        

        self.width = raster.RasterXSize
        self.height = raster.RasterYSize
        self.bands = raster.RasterCount
        
    def open(self, filename, mode):
        raster = gdal.Open(filename, gdal.GA_ReadOnly if mode == Raster.READ else gdal.GA_Update)

        if raster is None:
            raise Exception("O arquivo espeficado para o raster não existe ou não é um tipo válido")
        
        return raster

    def dtype(self):
        return self._raster.GetRasterBand(1).DataType

    def close(self):
        self._raster = None

    def transform(self, transform=None):
        if transform is None:
            return self._raster.GetGeoTransform()
        else:
            self._raster.SetGeoTransform(transform)

    def epsg(self, epsg = None):
        if epsg is None:
            srs = osr.SpatialReference()
            srs.ImportFromWkt(self._raster.GetProjection())
            return self.__toInt(srs.GetAttrValue('AUTHORITY',1))
        else:
            srs = osr.SpatialReference()
            srs.ImportFromEPSG(epsg)
            self._raster.SetProjection(srs.ExportToWkt())

    def geoReference(self, geoReference=None):
        if geoReference is None:
            return GeoReference.createFromTransform(self.transform(), self.width, self.height)
        
        self.transform(geoReference.transform())
        
    def write(self, array, order=RasterOrder.RGBC):
        for i in range(self.bands):
            if order == RasterOrder.CRGB:
                band = array[i, :,:]
                self._raster.GetRasterBand(i + 1).WriteArray(band)
            else:
                band = array[:,:, i]
                self._raster.GetRasterBand(i + 1).WriteArray(band)

    def envelope(self):
        geo = self.geoReference()

        p1 = geo.pivot()
        p2 = geo.pointToLatLng((self._raster.RasterXSize, 0))
        p3 = geo.pointToLatLng((self._raster.RasterXSize, self._raster.RasterYSize))
        p4 = geo.pointToLatLng((0, self._raster.RasterYSize))

        return Envelope(p1, p2, p3, p4)

    def extent(self):
        geoRef = self.geoReference()
        pivot = geoRef.pivot()

        se = geoRef.pointToLatLng((self._raster.RasterXSize, self._raster.RasterYSize))

        return Extent(pivot, se)

    def bbox(self):
        env = self.envelope()

        return Bbox(env.south_west(), env.north_east())

    def band(self, number):
        band = self._raster.GetRasterBand(number)

        return band.ReadAsArray()

    def clip(self, bbox:Bbox, size=256, order=RasterOrder.RGBC):
        raster = self._raster
        bands = raster.RasterCount
        querysize = size
        data = None

        p1 = bbox.first().toList()
        p2 = bbox.last().toList()

        bbox = (*p1, *p2)

        rb, wb = GeoMathUtils.query(raster, bbox[0], bbox[3], bbox[2], bbox[1], querysize)
        rx, ry, rxsize, rysize = rb
        wx, wy, wxsize, wysize = wb

        if rxsize != 0 and rysize != 0 and wxsize != 0 and wysize != 0:
            data = raster.ReadRaster(rx, ry, rxsize, rysize, wxsize, wysize, band_list=list(range(1, bands+1)), buf_type=self.dtype())

        if data:
            mem_drv = gdal.GetDriverByName('MEM')
            dsquery = mem_drv.Create('', querysize, querysize, bands, eType=self.dtype())
            dsquery.WriteRaster(wx, wy, wxsize, wysize, data, band_list=list(range(1, bands+1)), buf_type=self.dtype())

            if order == RasterOrder.CRGB:
                array = np.zeros((bands, querysize, querysize), dtype=RasterType.convertToNumpyType(self.dtype()))

                for i in range(bands):
                    r = dsquery.GetRasterBand(i + 1)
                    array[i, :, :] = r.ReadAsArray(0, 0, querysize, querysize)

                return array
            else:
                array = np.zeros((querysize, querysize, bands), dtype=RasterType.convertToNumpyType(self.dtype()))

                for i in range(bands):
                    r = dsquery.GetRasterBand(i + 1)
                    array[:, :, i] = r.ReadAsArray(0, 0, querysize, querysize)

                return array

        return None

    def crop(self, roi=None, order=RasterOrder.RGBC):
        """
        Retorna um array com as bandas do raster

        Args:
            roi (tuple) (opcional): Tupla contendo os valores (origemX, origemY, width, height)

        Returns:
            numpy.array: Array com todas as bandas contidas no raster
        """
        bands = self.bands

        rasterXSize = self._raster.RasterXSize if roi is None else roi[2]
        rasterYSize = self._raster.RasterYSize if roi is None else roi[3]

        if order == RasterOrder.CRGB:
            array = np.zeros((bands, rasterYSize, rasterXSize), dtype=RasterType.convertToNumpyType(self.dtype()))
        else:
            array = np.zeros((rasterYSize, rasterXSize, bands), dtype=RasterType.convertToNumpyType(self.dtype()))

        for i in range(bands):
            r = self._raster.GetRasterBand(i + 1)
            if order == RasterOrder.CRGB:
                if roi is None:
                    array[i, :, :] = r.ReadAsArray(0, 0, rasterXSize, rasterYSize)
                else:
                    array[i, :, :] = r.ReadAsArray(roi[0], roi[1], roi[2], roi[3])
            else:
                if roi is None:
                    array[:, :, i] = r.ReadAsArray(0, 0, rasterXSize, rasterYSize)
                else:
                    array[:, :, i] = r.ReadAsArray(roi[0], roi[1], roi[2], roi[3])

        return array
        
    def clone(self, filename):
        dtype = self._raster.GetRasterBand(1).DataType
        driverName = self._raster.GetDriver().GetDescription()

        clone = Raster.create(filename, self.width, self.height, self.bands, dtype, driverName)
        clone.epsg(self.epsg())
        clone.transform(self.transform())

        return clone

    @staticmethod
    def create(filename, xsize, ysize, bands, dtype, driverName='GTiff'):
        driver = gdal.GetDriverByName(driverName)
        driver.Create(filename, xsize=xsize, ysize=ysize, bands=bands, eType=dtype)

        return Raster(filename, Raster.WRITE)
  
    def __toInt(self, str:str):
        try:
            return int(str)
        except Exception as ex:
            return 0



import xml.etree.ElementTree as ET

from osgeo import ogr
from pytezergis.raster.abstract import AbstractRaster

class KML(AbstractRaster):
    def __init__(self, filename:str):
        super(KML, self).__init__(filename)

    def open(self, filename:str):
        driver = ogr.GetDriverByName('KML')
        self._filename = filename
        return driver.Open(filename)
    
    def metadata(self):
        xml_data = ''
        with open(self._filename, "r") as file:
            xml_data = file.read()

        # Parse do XML
        root = ET.fromstring(xml_data)

        # Namespace utilizado no XML
        namespace = "{http://www.opengis.net/kml/2.2}"

        # Dicionário para armazenar os dados por SchemaData
        metadata = []

        # Itera sobre todos os elementos <SchemaData> dentro do XML
        for schema_data in root.findall(f".//{namespace}SchemaData"):
            schema_id = schema_data.get("schemaUrl")
            simple_data_list = {}
            
            # Itera sobre todos os elementos <SimpleData> dentro de cada <SchemaData>
            for simple_data in schema_data.findall(f".//{namespace}SimpleData"):
                name = simple_data.get("name")
                value = simple_data.text
                simple_data_list[name] = value
            
            metadata.append(simple_data_list)

        return metadata


from pytezergis.spatial.latlng import Latlng

class Envelope:
    def __init__(self, p1:Latlng, p2:Latlng, p3:Latlng, p4:Latlng):
        """
        Classe que manipula 4 coordenadas representando os limites de uma geometria

        Args:
            p1 (tuple(Latlng), Latlng)
            p2 (Latlng, optional)
            p3 (Latlng, optional)
            p4 (Latlng, optional)
        """

        self.p1:Latlng = p1
        self.p2:Latlng = p2
        self.p3:Latlng = p3
        self.p4:Latlng = p4

    def north_east(self):
        """
        Retorna o ponto mais à direita superior
        """
        p = self.p1

        if self.p2.lat >= p.lat and self.p2.lng >= p.lng:
            p = self.p2
        
        if self.p3.lat >= p.lat and self.p3.lng >= p.lng:
            p = self.p3

        if self.p4.lat >= p.lat and self.p4.lng >= p.lng:
            p = self.p4

        return p

    def south_west(self):
        """
        Retorna o ponto mais à esquerda inferior
        """
        p = self.p1

        if self.p2.lat <= p.lat and self.p2.lng <= p.lng:
            p = self.p2
        
        if self.p3.lat <= p.lat and self.p3.lng <= p.lng:
            p = self.p3

        if self.p4.lat <= p.lat and self.p4.lng <= p.lng:
            p = self.p4

        return p

    def south_east(self):
        """
        Retorna o ponto mais à direita inferior
        """
        p = self.p1

        if self.p2.lat <= p.lat and self.p2.lng >= p.lng:
            p = self.p2
        
        if self.p3.lat <= p.lat and self.p3.lng >= p.lng:
            p = self.p3

        if self.p4.lat <= p.lat and self.p4.lng >= p.lng:
            p = self.p4

        return p

    def pivot(self):
        """
        Retorna o ponto mais à esquerda superior
        """
        p = self.p1

        if self.p2.lat >= p.lat and self.p2.lng <= p.lng:
            p = self.p2
        
        if self.p3.lat >= p.lat and self.p3.lng <= p.lng:
            p = self.p3

        if self.p4.lat >= p.lat and self.p4.lng <= p.lng:
            p = self.p4

        return p

    def isIntersect(self, envelope):
        latlower = 0
        lathigh = 0

        lnglower = 0
        lnghigh = 0

        if self.p1.lat < self.p4.lat:
            latlower = self.p1.lat
            lathigh = self.p4.lat
        else:
            latlower = self.p4.lat
            lathigh = self.p1.lat

        if self.p1.lng < self.p2.lng:
            lnglower = self.p1.lng
            lnghigh = self.p2.lng
        else:
            lnglower = self.p2.lng
            lnghigh = self.p1.lng

        if envelope.p1.lat >= latlower and envelope.p1.lat <= lathigh:
            if envelope.p1.lng >= lnglower and envelope.p1.lng <= lnghigh:
                return True

        if envelope.p2.lat >= latlower and envelope.p2.lat <= lathigh:
            if envelope.p2.lng >= lnglower and envelope.p2.lng <= lnghigh:
                return True

        if envelope.p3.lat >= latlower and envelope.p3.lat <= lathigh:
            if envelope.p3.lng >= lnglower and envelope.p3.lng <= lnghigh:
                return True

        if envelope.p4.lat >= latlower and envelope.p4.lat <= lathigh:
            if envelope.p4.lng >= lnglower and envelope.p4.lng <= lnghigh:
                return True

        return False

    def toList(self):
        return (
            (self.p1.lng, self.p1.lat),
            (self.p2.lng, self.p2.lat),
            (self.p3.lng, self.p3.lat),
            (self.p4.lng, self.p4.lat),
        )

    def toWkt(self):
        return "POLYGON ((%s %s, %s %s, %s %s, %s %s, %s %s))" % (
            self.p1.lng, self.p1.lat,
            self.p2.lng, self.p2.lat,
            self.p3.lng, self.p3.lat,
            self.p4.lng, self.p4.lat,
            self.p1.lng, self.p1.lat,
        )
    
    def __str__(self):
        return self.toWkt()


class Latlng:
    def __init__(self, lat:float, lng:float):
        """
        Classe representando uma coordenada de ponto

        Args:
            lat (float)
            lng (float)
        """
        self.lat = lat
        self.lng = lng

    def toList(self):
        return (self.lng, self.lat)

    def compareTo(self, latlng):
        if self.lat < latlng.lat and self.lng < latlng.lng:
            return - 1
        elif self.lat == latlng.lat and self.lng == latlng.lng:
            return 0
        else:
            return 1
        
    def toWkt(self):
        return "POINT (%s %s)" % (self.lng, self.lat)
        
    def __getitem__(self, idx):
        return self.lng if idx == 0 else self.lat

    def __str__(self):
        return "POINT (%s %s)" % (self.lng, self.lat)


class AbstractRaster:
    def __init__(self, filename:str):
        """
        Classe abstrata representando um raster generico
        Caso o arquivo informado não existe ou não seja possivel abrir o raster será lançada uma excessão
        """
        self.dataSource = self.open(filename)
        if self.dataSource is None:
            raise Exception("Não foi possível abrir o raster informado")

    def features(self):
        features = []

        if self.dataSource is None:
            return features

        for i in range(self.dataSource.GetLayerCount()):
            layer = self.dataSource.GetLayerByIndex(i)
            for feature in layer:
                features.append({
                    'wkt': feature.GetGeometryRef().ExportToWkt(),
                    'fields': feature.items()
                })

        if len(features) == 0:
            layer = self.dataSource.GetLayer()
            for feature in layer:
                features.append({
                    'wkt': feature.GetGeometryRef().ExportToWkt(),
                    'fields': feature.items()
                })

        return features

    def geometries(self):
        geometries = []

        if self.dataSource is None:
            return geometries

        for i in range(self.dataSource.GetLayerCount()):
            layer = self.dataSource.GetLayerByIndex(i)
            for feature in layer:
                geom = feature.GetGeometryRef()
                geometries.append(geom.ExportToWkt())

        if len(geometries) == 0:
            layer = self.dataSource.GetLayer()
            for feature in layer:
                geometries.append(feature.GetGeometryRef().ExportToWkt())

        return geometries

    def fields(self):
        arr = []

        if self.dataSource is None:
            return arr

        layers = self.dataSource.GetLayerCount()

        # Loop através das camadas
        for i in range(layers):
            layer = self.dataSource.GetLayer(i)

            # Loop através das feições (geometrias) na camada
            for feature in layer:
                # Obtenha as informações dos atributos (chave-valor) da feição
                arr.append(feature.items())

        return arr
    
    def epsg(self, type:str='GEOGCS'):
        if self.dataSource is None:
            return None
    
        layer= self.dataSource.GetLayer()
        crs = layer.GetSpatialRef()

        return crs.GetAuthorityCode(type)

    def open(self, filename):
        return None

    def close(self):
        self.dataSource = None


import ogr

from pytezergis.spatial.polygon import Polygon

class Wkt:
    def __init__(self, wkt):
        self.wkt = wkt
        self.geom = ogr.CreateGeometryFromWkt(self.wkt)

    @staticmethod
    def createFromWkb(wkb):
        geometry = ogr.CreateGeometryFromWkb(wkb)
        wkt = geometry.ExportToWkt()
        return Wkt(wkt)

    def isValid(self):
        return self.geom is not None

    def envelope(self):
        polygon = Polygon(self.wkt)
        return polygon.envelope()
    
    def isPoint(self):
        geom = self.geom
        geomType = geom.GetGeometryType()
        return geomType == ogr.wkbPoint or geomType == ogr.wkbPointM or geomType == ogr.wkbPointZM or geomType == ogr.wkbPoint25D

    def isMultiPoint(self):
        geom = self.geom
        geomType = geom.GetGeometryType()
        return geomType == ogr.wkbMultiPoint or geomType == ogr.wkbMultiPointM or geomType == ogr.wkbMultiPointZM or geomType == ogr.wkbMultiPoint25D
    
    def isLineString(self):
        geom = self.geom
        geomType = geom.GetGeometryType()
        return geomType == ogr.wkbLineString or geomType == ogr.wkbLineStringM or geomType == ogr.wkbLineStringZM or geomType == ogr.wkbLineString25D
    
    def isMultiLineString(self):
        geom = self.geom
        geomType = geom.GetGeometryType()
        return geomType == ogr.wkbMultiLineString or geomType == ogr.wkbMultiLineStringM or geomType == ogr.wkbMultiLineStringZM or geomType == ogr.wkbMultiLineString25D

    def isPolygon(self):
        geom = self.geom
        geomType = geom.GetGeometryType()
        return geomType == ogr.wkbPolygon or geomType == ogr.wkbPolygonM or geomType == ogr.wkbPolygonZM or geomType == ogr.wkbPolygon25D

    def isGeometryCollection(self):
        geom = self.geom
        geomType = geom.GetGeometryType()
        return geomType == ogr.wkbGeometryCollection or geomType == ogr.wkbGeometryCollectionM or geomType == ogr.wkbGeometryCollectionZM or geomType == ogr.wkbGeometryCollection25D

    def isCircularString(self):
        geom = self.geom
        geomType = geom.GetGeometryType()
        return geomType == ogr.wkbCircularString or geomType == ogr.wkbCircularStringZ or geomType == ogr.wkbCircularStringM or geomType == ogr.wkbCircularStringZM or geomType == ogr.wkbCircularString25D

    def isMultiPolygon(self):
        geom = self.geom
        geomType = geom.GetGeometryType()
        return geomType == ogr.wkbMultiPolygon or geomType == ogr.wkbMultiPolygonM or geomType == ogr.wkbMultiPolygonZM or geomType == ogr.wkbMultiPolygon25D

    def isLinearRing(self):
        geom = self.geom
        geomType = geom.GetGeometryType()
        return geomType == ogr.wkbLinearRing


from setuptools import setup, find_packages

setup(
    name='pytezergis',
    version='1.2',
    packages=find_packages(),
    description='Pacote responsável pela manipulação de dados geospaciais',
    author='Manoel Neco Jr',
    author_email='necoengenheiro@gmail.com',
)



from osgeo import ogr, osr

from pytezergis.spatial.extent import Extent
from pytezergis.spatial.latlng import Latlng

class LineString:

    def __init__(self, wkt, epsg=4326):
        """
        Classe representando uma linestring

        Args:
            wkt (str, list(Latlng))
            epsg (int, optional)
        """

        if isinstance(wkt, (list, tuple)):
            wkt = self.__latlngsToWkt(wkt)

        self.wkt = wkt
        self.epsg = epsg

        self.geom = ogr.CreateGeometryFromWkt(wkt)
    
    def isValid(self):
        return self.geom is not None

    def envelope(self):
        env = self.geom.GetEnvelope()
        return Extent.createFromList((env[0], env[2], env[1], env[3])).toEnvelope()

    def clone(self):
        return LineString(self.wkt, self.epsg)

    def buffer(self, distance, epsg=4326):
        poly = self.clone()

        if poly.epsg != epsg:
            poly.transform(epsg)
        
        poly.geom = poly.geom.Buffer(distance)
        poly.wkt = poly.geom.ExportToWkt()

        if poly.epsg != self.epsg:
            poly.transform(self.epsg)

        return poly

    def perimeter(self):
        return self.geom.Boundary().Length()

    def transform(self, epsg):
        src = osr.SpatialReference()
        src.ImportFromEPSG(self.epsg)

        dst = osr.SpatialReference()
        dst.ImportFromEPSG(epsg)

        transform = osr.CoordinateTransformation(src, dst)

        self.geom.Transform(transform)
        self.wkt = self.geom.ExportToWkt()

        self.epsg = epsg

    def toLatlngs(self):
        arr = []

        for i in range(0, self.geom.GetPointCount()):
            p = self.geom.GetPoint(i)
            arr.append(Latlng(p[1], p[0]))

        return arr

    def toWkt(self):
        return self.wkt

    def __latlngsToWkt(self, latlngs):
        ring = ogr.Geometry(ogr.wkbLinearRing)

        for latlng in latlngs:
            ring.AddPoint(latlng.lng, latlng.lat)

        return ring.ExportToWkt()

    def __str__(self):
        return self.wkt


from pytezergis.raster.kml import KML

from pytezerio.directory import Directory
from pytezerio.file import File

from zipfile import ZipFile

class KMZ(KML):
    def __init__(self, filename:str):
        super(KMZ, self).__init__(filename)

    def open(self, filename):
        self.__temp = Directory.temporary()
        with ZipFile(filename, 'r') as zip:
            zip.extractall(self.__temp.fullpath)

        for filename in self.__temp.list(Directory.FILES):
            file = File(filename)
            if file.extension() == 'kml':
                return super().open(self.__temp.resolve(filename))

    def __del__(self):
        self.close()

        if self.__temp is not None:
            self.__temp.remove()
        
        self.__temp = None
